#ifndef RT_GFX_RENDER_VIEW_H
#define RT_GFX_RENDER_VIEW_H

/* A render view acts as a container of one or more render lists.
 * Each view is processed by exactly one pass. */

#include "render_list.h"

typedef struct rt_render_graph_s rt_render_graph;

typedef struct {
    rt_render_list *lists;
    rt_render_object_type *list_types;
    uint32_t list_count;
    rt_render_object_type_mask type_mask;
} rt_render_view;

typedef struct {
    bool ok;
    rt_render_view view;
} rt_create_render_view_result;

#ifdef __cplusplus
extern "C" {
#endif

RT_DLLEXPORT rt_create_render_view_result rtCreateRenderView(const rt_render_object_type *types,
                                                             uint32_t type_count,
                                                             unsigned int frame_id);

RT_DLLEXPORT bool
rtPushRenderObjectToView(rt_render_view *view, rt_render_object_type type, const void *object);

RT_DLLEXPORT void rtSubmitRenderView(rt_render_view view,
                                     rt_render_graph *render_graph,
                                     uint32_t pass_id,
                                     unsigned int frame_id);

/* Checks if the view contains exactly the given types in the given order */
RT_DLLEXPORT int rtDoViewTypesMatchExact(rt_render_view view,
                                         const rt_render_object_type *types,
                                         uint32_t type_count);

/* Checks if the view contains exactly the given types, in any order */
RT_INLINE static int
rtDoViewTypesMatch(rt_render_view view, const rt_render_object_type *types, uint32_t type_count) {
    rt_render_object_type_mask in_mask = 0;
    for (uint32_t i = 0; i < type_count; ++i) {
        in_mask |= RT_RENDER_OBJECT_TYPE_BIT(types[i]);
    }
    return view.type_mask == in_mask;
}

RT_INLINE static int rtDoesViewContainTypes(rt_render_view view,
                                            rt_render_object_type_mask type_mask) {
    return (int)(view.type_mask & type_mask);
}

#ifdef __cplusplus
}
#endif

#endif
